/* REXX Exec */
ver = "1.2"
/*
   Name: SHAREPDA.REX
   Function: To convert a csv exported SHARE Schedule into a Palm OS
             Calendar entry in the format of the vCalendar file and
	     create a iCalendar file for use with Lotus Notes or 
	     Outlook.
	  
   Syntax: REXX SHAREPDA.REX
   
   The user will be prompted for several required pieces of information.
   
   Prompt 1: the name of the SHARE Scheduler exported csv file.
   The first record must contain the headers of each field.

   The fields should be:
   
   EventDate,StartTime,Duration,Title,Locale,Abstract,Speaker,Chair
   
   Prompt 2: option to flag speaker and chair sessions by entering 
             a name. this name will be tested in both speaker and chair
	     fields and if found will append to the start of the title
	     *S* for speaker, *C* for chair, or *SC* for both.
   
   Notes: This code is written in Regina Rexx and should work on any
   	  platform that supports Rexx with Lines, Linein, and Linevout.
   
   History:
   	01/19/2004  1.2 Remove extraneous SAY
	01/13/2004  1.1 Correction for very long sessions (end time)
	12/13/2003  1.0	Add Processing Completed message and set as version 1.0
   	10/02/2003  0.9 Correct flag for *SC*
	09/02/2003  0.8 Finally looks like it fully support both vcs and 
	                ics files.
	09/01/2003  0.7 Add iCalendar output file
	08/17/2003  0.6 Remove UTC prompt and remove Z (zulu) indicator
		        from date/time as without the Z the timezone is
			the current timezone.
			Add version to message.
			Add SHARE copyright.
	08/14/2003  0.5 Correct for multiple speakers
	08/09/2003  0.4 Improve format of the Abstract text
	08/07/2003  0.3 Remove delete statements and use lineout with
		        record 1 to replace existing files
	08/07/2003  0.2 Additional cleanup and enhancement
   	08/06/2003  0.1	Creation
	
   Author:
   	Lionel B. Dyck
	Kaiser Permanente Information Technology
	25 N. Via Monte Ave
	Walnut Creek, Ca 94598
	e-mail: lionel.b.dyck@kp.org
	
   Copyright 2003 SHARE Inc. All Rights Reserved
   Licensed under the GNU Public License (GPL)
*/

Say " "
Say " "
Say "You are running the SHARE Scheduling Tool Palm Desktop",
    "Post Processor version" ver"."
Say "You will need to Export your SHARE Schedule with the following"
Say "fields checked:"
Say " "
Say "EventDate,StartTime,Duration,Title,Locale,Abstract,Speaker,Chair"
Say " "
Say "You may also check Alternate Sessions if you desire."
Say " "
Say "If you are not ready then do your export now and return to"
Say "this post processor."
Say " "
Say "This package will generate both vCalendar and iCalendar files."
Say " "

Say "Enter the name of the SHARE Scheduler Exported CSV File:"
Pull input_csv

If strip(input_csv) = "" then do
	Say "Error."
	Say "You were asked to enter the name of a csv file and",
	    "you did not."
	Say "Therefore this command will exit."
	Exit 8
	end

if lines(input_csv) = 0 then do
	Say "It appears that your input file" input_csv ,
	    "is empty. Try again."
	Exit 8
	end

Say "Enter a name to check to flag Speaker or Chair sessions. This"
Say "is typically for those speaking and chairing:"
Pull flag_name
flag_name = translate(flag_name)

/* Set the initial counter for the output records 
   and set the initial records. */
vc     = 3
vout.1 = "BEGIN:VCALENDAR"
vout.2 = "VERSION:1.0"
vout.3 = "PRODID:SHARE Scheduler PostProcessor Generated"

ic     = 4
iout.1 = "BEGIN:VCALENDAR"
iout.2 = "METHOD: TEXT"
iout.3 = "VERSION:2.0"
iout.4 = "PRODID:SHAREPDA.REX" ver "//NONSGML SHARE//EN"

/* create the output file names */
parse value input_csv with file"." .
out_vcs = file".vcs"
out_ics = file".ics"

null = ""

rec = linein(input_csv)
fields = "EventDate,StartTime,Duration,Title,Locale,Abstract,Speaker,Chair"
if rec <> fields then do
	     Say "The input file does not conform to the expected format."
	     Say "The first record must contain the field names."
	     Say "The fields expected are:"
	     Say fields
	     Say "Exiting - return to the SHARE Scheduling tool and",
	         "try again."
	     Exit 8
	     end	     
			     
/* Process fields:
   EventDate,StartTime,Duration,Title,Locale,Abstract,Speaker,Chair
*/
icount = 0
Do forever
	if lines(input_csv) = 0 then leave
	rec = linein(input_csv)
	icount = icount + 1
	in.icount = rec
	end

/* quick fix by adding 1 blank record at the end */
icount = icount + 1
in.icount = " "

do icnt = 1 to icount
   call Add_Line
   end

/* Now close the input file */
call lineout(input_csv)

vc = vc + 1
vout.vc = " "
vc = vc + 1
vout.vc = "END:VCALENDAR"

ic = ic + 1
iout.ic = "END:VCALENDAR"

/* write the generated vCal and iCal files out */
x = lineout(out_vcs,vout.1,1)
do i = 2 to vc
   x = lineout(out_vcs,vout.i)
   end

x = lineout(out_ics,iout.1,1)
do i = 2 to ic
   x = lineout(out_ics,iout.i)
   end

/* now close the output files */   
call lineout out_vcs
call lineout out_ics

Say "Schedule file:" input_csv "has been converted to vCalendar",
    "format"
Say "and into iCalendar format."
Say "Import into your Palm Desktop file:" out_vcs
Say "Import into Lotus Notes or Outlook the file:" out_ics
Say " "
Say "Processing Completed" date() "at" time()

Exit

/* Routine to add the current record to the vCal and iCal stems */
Add_Line:
call Add_vCal
call Add_iCal
return

/* Routine to add the current record to the vCal output stem */
Add_vCal:
rec = strip(in.icnt)
reci = rec

do until t = icount
	 t = icnt + 1
	 if left(in.t,1) = '"' then leave
	 icnt = icnt + 1
	 rec = strip(rec)"=0D=0A"strip(in.icnt)
	 reci = strip(reci)"\n"strip(in.icnt)
	 end
	 
parse value rec with '"'date'",'rec
parse value rec with '"'time'",'rec
parse value rec with '"'dur'",'rec
parse value rec with '"'title'",'rec
parse value rec with '"'loc'",'rec
parse value rec with '"'abs'",'rec
parse value rec with '"'speak'",'rec
parse value rec with '"'chair'"'.

Say "vCal Processing Session:" title

if flag_name <> "" then do
   flag = ""
   if pos(flag_name,translate(speak)) > 0 then
      flag = "*S*"
   if pos(flag_name,translate(chair)) > 0 then
      flag = "*C*"
   if flag <> "" then 
      title = flag""title
   end

if strip(loc) <> "" then
   title = strip(title"=0D=0ALocation:" loc)

vc = vc + 1
vout.vc = "BEGIN:VEVENT"
vc = vc + 1
vout.vc = "SUMMARY;ENCODING=QUOTED-PRINTABLE:"title
desc = ""
if strip(speak) <> "" then
   desc = "Speaker:" speak "=0D=0A=0D=0A"
if strip(chair) <> "" then
   desc = desc"Chair:" chair "=0D=0A=0D=0A"
if strip(loc) <> "" then
   desc = desc"Location:" loc "=0D=0A=0D=0A"
desc = desc "=0D=0A"
desc = desc""abs
vc = vc + 1
vout.vc = "DESCRIPTION;ENCODING=QUOTED-PRINTABLE:"desc

/* Create the DTSTART and DTEND statements */

parse var date mm"/"dd"/"yyyy
edate = yyyy""right(mm+100,2)""right(dd+100,2)
parse var time hh":"mm
etime = right(hh+100,2)right(mm+100,2)
etime = right(etime + 10000,4)
vc = vc + 1
vout.vc = "DTSTART:"edate"T"etime"00"

hh = left(etime,2)
mm = right(etime,2)

/* get duration in hours */
durh = (dur % 60) + hh
/* get minutes */
durm = (dur // 60) + mm

if durm > 60 then do
   durh = durh + (durm / 60)
   durm = durm // 60
   end
   

hh = right(durh+100,2)
mm = right(durm+100,2)

etime = hh""mm 

vc = vc + 1
vout.vc = "DTEND:"edate"T"etime"00"
vc = vc + 1
vout.vc = "END:VEVENT"
return

/* --------------------------------------------------------- *
   Routine to add the current record to the iCal output stem *
   --------------------------------------------------------- */
Add_iCal:
rec = reci
parse value rec with '"'date'",'rec
parse value rec with '"'time'",'rec
parse value rec with '"'dur'",'rec
parse value rec with '"'title'",'rec
parse value rec with '"'loc'",'rec
parse value rec with '"'abs'",'rec
parse value rec with '"'speak'",'rec
parse value rec with '"'chair'"'.

speak = strip(speak)
chair = strip(chair)
loc   = strip(loc)

Say "iCal Processing Session:" title

if flag_name <> "" then do
   flag = ""
   if pos(flag_name,translate(speak)) > 0 then
      flag = "*S*"
   if pos(flag_name,translate(chair)) > 0 then
      if flag <> "" 
      	then flag = "*SC*"
      	else flag = "*C*"
   if flag <> "" then 
      title = flag""title
   end

ic = ic + 1
iout.ic = "BEGIN:VEVENT"
ic = ic + 1
iout.ic = "SUMMARY:"title

/* Create the DTSTART and DURATION statements */

parse var date mm"/"dd"/"yyyy
edate = yyyy""right(mm+100,2)""right(dd+100,2)
parse var time hh":"mm
etime = right(hh+100,2)right(mm+100,2)
etime = right(etime + 10000,4)
ic = ic + 1
iout.ic = "DTSTART:"edate"T"etime"00"
ic = ic + 1
iout.ic = "DURATION:PT"dur"M"

ic = ic + 1
iout.ic = "LOCATION:" loc

if chair <> "" then do
   ic = ic + 1
   iout.ic = "ORGANIZER;ROLE=CHAIR:"chair
   end

desc = left("DESCRIPTION: ",75)
ic = ic + 1
iout.ic = desc
if speak <> null then do
   ic = ic + 1
   iout.ic = strip(left(" Speaker:"speak,73),'t')"\n"
   ic = ic + 1
   iout.ic = " \n"
   end
if chair <> null then do
   ic = ic + 1
   iout.ic = strip(left(" Chair:" chair,73),'t')"\n"
   ic = ic + 1
   iout.ic = " \n"
   end
if loc <> null then do
   ic = ic + 1
   iout.ic = strip(left(" Location:"loc,73),'t')"\n"
   ic = ic + 1
   iout.ic = " \n"
   end

do until pos("=0D=0A",abs) = 0
   parse value abs with absl"=0D=0A"absr
   abs = absl"\n"absr
   end

do until length(abs) = 0
   parse value abs with desc"\n"abs
   do until length(desc) = 0
      descl = left(desc,73)
      desc  = substr(desc,74)
      ic = ic + 1
      iout.ic = " "descl
      end
   ic = ic + 1
   iout.ic = " \n"
   end   

ic = ic + 1
iout.ic = "END:VEVENT"

return

exit 